from . import base64_task
from . import env_task
from . import eval_task
from . import md5_task
from . import principle_task
from . import ubuntu_task
from .project import project_create_task
from .task import task_create_task

assert base64_task
assert env_task
assert eval_task
assert md5_task
assert principle_task
assert ubuntu_task
assert project_create_task
assert task_create_task
